from sksurv import compare, util
import numpy as np


def _find_split(node):
    """
    Find the best split for a Node.
    :param node: Node to find best split for.
    :return: score of best split, value of best split, variable to split, left indices, right indices.
    """
    score_opt = 0
    split_val_opt = None
    lhs_idxs_opt = None
    rhs_idxs_opt = None
    split_var_opt = None
    for i in node.f_idxs:
        score, split_val, lhs_idxs, rhs_idxs = _find_best_split_for_variable(node, i)

        if score > score_opt:
            score_opt = score
            split_val_opt = split_val
            lhs_idxs_opt = lhs_idxs
            rhs_idxs_opt = rhs_idxs
            split_var_opt = i

    return score_opt, split_val_opt, split_var_opt, lhs_idxs_opt, rhs_idxs_opt


def _find_best_split_for_variable(node, var_idx):
    """
    Find best split for a variable of a Node. Best split for a variable is the split with the highest log rank
    statistics. The logrank_test function of the lifelines package is used here.
    :param node: Node
    :param var_idx: Index of variable
    :return: score, split value, left indices, right indices.
    """
    score, split_val, lhs_idxs, rhs_idxs = logrank_statistics(x=node.x, y=node.y,
                                                              feature=var_idx,
                                                              min_leaf=node.min_leaf)
    return score, split_val, lhs_idxs, rhs_idxs


def logrank_statistics(x, y, feature, min_leaf):
    """
    Compute logrank_test of liflines package.
    :param x: Input samples
    :param y: Labels
    :param feature: Feature index
    :param min_leaf: Minimum number of leafs for each split.
    :return: best score, best split value, left indices, right indices
    """
    x_feature = x.reset_index(drop=True).iloc[:, feature]
    score_opt = 0
    split_val_opt = None
    lhs_idxs = None
    rhs_idxs = None

    unique_vals = np.sort(np.unique(x_feature))

    for split_val in unique_vals:
        feature1 = np.where(x_feature <= split_val)[0]
        feature2 = np.where(x_feature > split_val)[0]

        if len(feature1) < min_leaf or len(feature2) < min_leaf:
            continue

        y_tmp = y.iloc[np.concatenate((feature1, feature2)), :2]
        y_tmp['group_indicator'] = np.nan
        y_tmp.iloc[feature1, -1] = 0
        y_tmp.iloc[feature2, -1] = 1

        try:
            score, p_value = compare.compare_survival(
                util.Surv.from_dataframe("arrest", "week", y_tmp.drop(columns='group_indicator')),
                y_tmp.loc[:, 'group_indicator'], return_stats=False)
        except ValueError:
            continue
        if score > score_opt:
            score_opt = round(score, 3)
            split_val_opt = round(split_val, 3)
            lhs_idxs = feature1
            rhs_idxs = feature2

    return score_opt, split_val_opt, lhs_idxs, rhs_idxs



import time

from lifelines import datasets
from sklearn.model_selection import train_test_split

from random_survival_forest.models import RandomSurvivalForest
from random_survival_forest.scoring import concordance_index

rossi = datasets.load_rossi()
# Attention: duration column must be index 0, event column index 1 in y
y = rossi.loc[:, ["arrest", "week"]]
X = rossi.drop(["arrest", "week"], axis=1)
X, X_test, y, y_test = train_test_split(X, y, test_size=0.33, random_state=10)

print("Start training...")
start_time = time.time()
rsf = RandomSurvivalForest(n_estimators=50, n_jobs=-1, random_state=42)
rsf = rsf.fit(X, y)
print(f'--- {round(time.time() - start_time, 3)} seconds ---')
y_pred = rsf.predict(X_test)
c_val = concordance_index(y_time=y_test["week"], y_pred=y_pred, y_event=y_test["arrest"])
print(f'C-index {round(c_val, 3)}')



from setuptools import setup

with open('README.md') as f:
    readme = f.read()

setup(
    name='random_survival_forest',
    packages=['random_survival_forest'],
    version='0.8.2',
    license="MIT License",
    long_description=readme,
    long_description_content_type="text/markdown",
    description='A Random Survival Forest implementation inspired by Ishwaran et al.',
    author='Julian Späth',
    author_email='spaethju@posteo.de',
    url='https://github.com/julianspaeth/random-survival-forest',
    download_url='https://github.com/julianspaeth/random-survival-forest/archive/v0.1-beta.tar.gz',
    keywords=['survival-analysis', 'survival-prediction', 'machine-learning', 'random-forest',
              'random-survival-forest'],
    install_requires=[
        'numpy',
        'pandas',
        'joblib',
        'multiprocess',
        'lifelines',
        'scikit-learn',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Software Development :: Build Tools', 'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)
